#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.test import TestCase
from django.test.utils import override_settings
from mock import Mock

from holonet_django import handlers, validators
from holonet_django.exceptions import HolonetConfigrationError
from holonet_django.signals import (add_signal_listeners, pre_save_mapping_check,
                                    pre_save_recipient_check)
from tests.models import Mapping1, Mapping2, TestRecipientModel


class SignalsTestCase(TestCase):

    def test_recipient_signal(self):

        handlers.handle_recipient_change = Mock()

        test_object = TestRecipientModel(username='test', email='test@holonet.no')
        test_object.save()

        handlers.handle_recipient_change.assert_called_with(test_object, True, None)

    def test_mapping_signal(self):
        handlers.handle_mapping_change = Mock()

        mapping1 = Mapping1(mail_prefix='holonet_test')
        mapping1.save()

        mapping2 = Mapping2(mail_prefix='holonet_test')
        mapping2.save()

        handlers.handle_mapping_change.assert_any_call(mapping1, True, None)
        handlers.handle_mapping_change.assert_any_call(mapping2, True, None)
        self.assertEquals(handlers.handle_mapping_change.call_count, 2)

    def test_recipient_delete_signal(self):
        handlers.handle_recipient_delete = Mock()

        test_object = TestRecipientModel(username='test', email='test@holonet.no')
        test_object.save()
        test_object.delete()

        handlers.handle_recipient_delete.assert_called_once(test_object)

    def test_mapping_recipient_list_change(self):
        mapping1 = Mapping1(mail_prefix='holonet_test')
        mapping1.save()

        test_object = TestRecipientModel(username='test', email='test@holonet.no')
        test_object.save()

        mapping1.recipients.add(test_object)

    def test_mapping_delete_signal(self):
        handlers.handle_mapping_delete = Mock()

        mapping = Mapping1(mail_prefix='holonet_test')
        mapping.save()

        mapping.delete()

        handlers.handle_mapping_delete.assert_called_once_with(mapping)

    @override_settings(HOLONET_MAPPING_MODELS={
        'tests.Mapping1': {
            'recipient_relations': False
        }
    })
    def test_mapping_init_no_list(self):
        self.assertRaisesMessage(
            HolonetConfigrationError,
            'recipient_relations needs to be a list of strings.',
            add_signal_listeners
        )

    @override_settings(HOLONET_MAPPING_MODELS={
        'tests.Mapping1': {
            'recipient_relations': [False]
        }
    })
    def test_mapping_init_no_string_list(self):
        self.assertRaisesMessage(
            HolonetConfigrationError,
            'Each item in recipient_relations need to be a string.',
            add_signal_listeners
        )

    @override_settings(HOLONET_MAPPING_MODELS={
        'tests.Mapping1': {
            'recipient_relations': ['does_not_excist_field']
        }
    })
    def test_mapping_init_list_not_field(self):
        self.assertRaisesMessage(
            HolonetConfigrationError,
            'Could not find the relation_field on the model.',
            add_signal_listeners
        )

    @override_settings(HOLONET_MAPPING_MODELS={
        'tests.Mapping1': {
            'recipient_relations': ['invalid_test_field']
        }
    })
    def test_mapping_init_no_list_invalid_field(self):
        self.assertRaisesMessage(
            HolonetConfigrationError,
            'The relation field needs to be of type models.ManyToManyField',
            add_signal_listeners
        )

    def test_recipient_pre_validator(self):
        self.assertIsNone(pre_save_recipient_check(TestRecipientModel(email='test@holonet.no')))

        self.assertRaises(ValueError, pre_save_recipient_check, TestRecipientModel())

    def test_mapping_pre_validation(self):
        validators.is_prefix_unique = Mock(side_effect=lambda x, y: True)

        self.assertIsNone(pre_save_mapping_check(Mapping1(mail_prefix='holonet_test')))
        self.assertRaisesMessage(ValueError, 'The mail prefix is not valid.',
                                 pre_save_mapping_check, Mapping1())

        orginal = validators.is_prefix_unique
        validators.is_prefix_unique = Mock(side_effect=lambda x, y: False)

        mapping = Mapping1(mail_prefix='holonet_test1')
        self.assertRaisesMessage(ValueError, 'The selected mapping prefix is not unique.',
                                 pre_save_mapping_check, mapping)

        validators.is_prefix_unique = orginal



# -*- coding: utf8 -*-

from django.db.models import SlugField

from . import validators


class MailPrefixField(SlugField):

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 100
        kwargs['blank'] = False
        kwargs['null'] = False
        kwargs['unique'] = True
        kwargs['validators'] = [validators.validate_prefix]
        super(MailPrefixField, self).__init__(*args, **kwargs)



# -*- coding: utf8 -*-

from django.test import TestCase

from holonet_django.exceptions import HolonetConfigrationError
from holonet_django.handlers import (handle_mapping_change, handle_mapping_delete,
                                     handle_recipient_change, handle_recipient_delete)
from tests.models import TestRecipientModel


class HandlersTestCase(TestCase):

    def test_recipient_handler(self):
        self.assertRaises(HolonetConfigrationError, handle_recipient_change, None, False, None)

        test_object = TestRecipientModel(email='test@holonet.no')
        self.assertRaises(HolonetConfigrationError, handle_recipient_change, test_object, False,
                          None)  # No pk

        test_object.save()
        self.assertEquals(handle_recipient_change(test_object, True, None), (test_object.pk,
                                                                             test_object.email))

        test_object.email = 'test@holonet.no'
        self.assertEquals(handle_recipient_change(test_object, True, None), (test_object.pk,
                                                                             test_object.email))

    def test_mapping_handler(self):
        self.assertFalse(handle_mapping_change(None, False, None))

    def test_recipient_delete(self):

        test_object = TestRecipientModel(email='test@holonet.no')
        self.assertRaises(HolonetConfigrationError, handle_recipient_delete, test_object)

        test_object.save()
        self.assertEqual(handle_recipient_delete(test_object), test_object.pk)

    def test_mapping_delete(self):
        self.assertTrue(handle_mapping_delete(True))



# -*- coding: utf8 -*-
try:
    from django.core.exceptions import AppRegistryNotReady

    try:
        from holonet_django import signals
    except AppRegistryNotReady:
        default_app_config = 'holonet_django.apps.HolonetDjangoConfig'

except ImportError:
    AppRegistryNotReady = None
    from holonet_django import signals





#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django.test import TestCase
from django.test.utils import override_settings

from holonet_django.exceptions import HolonetConfigrationError
from holonet_django.settings import holonet_settings


class TestHolonetSettings(TestCase):

    def test_item_getter_no_setting(self):
        try:
            holonet_settings.ENDPOINT
        except HolonetConfigrationError as e:
            self.assertIsInstance(e, HolonetConfigrationError)

    @override_settings(HOLONET_ENDPOINT='test_endpoint')
    def test_setting_getter(self):
        self.assertEquals(holonet_settings.ENDPOINT, 'test_endpoint')



# -*- coding: utf8 -*-

from django.core.validators import validate_email as django_email_validate
from django.core.validators import EmailValidator, ValidationError


class PrefixValidator(EmailValidator):

    def __call__(self, prefix):
        value = '%s@holonet.no' % prefix
        super(PrefixValidator, self).__call__(value)

validate_prefix = PrefixValidator()


def validate_email(email):
    try:
        django_email_validate(email)
        return True
    except ValidationError:
        pass
    return False


def validate_local_part(local_part):
    return validate_email('%s@holonet.no' % local_part)


def is_prefix_unique(prefix, mapping_identifier):
    # Todo: Ask the api about this mapping uniqueness
    return True



# -*- coding: utf8 -*-

from .exceptions import HolonetConfigrationError
from .settings import holonet_settings


def handle_recipient_change(recipient, created, updated_fields):
    id = getattr(recipient, holonet_settings.RECIPIENT_UNIQUE_IDENTIFIER_FIELD, None)
    email = getattr(recipient, holonet_settings.RECIPIENT_EMAIL_FIELD, None)

    if id is None:
        raise HolonetConfigrationError('Holonet could not find the unique field %s.' %
                                       holonet_settings.RECIPIENT_UNIQUE_IDENTIFIER_FIELD)

    return id, email


def handle_mapping_change(mapping, created, updated_fields, force=False):
    return False


def handle_recipient_delete(recipient):
    identifier_field = getattr(recipient, holonet_settings.RECIPIENT_UNIQUE_IDENTIFIER_FIELD, None)

    if identifier_field is None:
        raise HolonetConfigrationError('Holonet could not find the unique field %s.' %
                                       holonet_settings.RECIPIENT_UNIQUE_IDENTIFIER_FIELD)

    return identifier_field


def handle_mapping_delete(mapping):
    return mapping



# -*- coding: utf8 -*-

import sys

from django.conf import settings

from .exceptions import HolonetConfigrationError


class Settings(object):

    CONFIGS = {
        'API_ENDPIONTS': {
            'sender_blacklist': '/sender-blacklist/',
            'domain_blacklist': '/domain-blacklist/',
        },
        'TESTING': 'test' in sys.argv,
        'RECIPIENT_MODEL': settings.AUTH_USER_MODEL,
        'RECIPIENT_UNIQUE_IDENTIFIER_FIELD': 'pk',
        'RECIPIENT_EMAIL_FIELD': 'email',
        'MAPPING_MODELS': {
        }
    }

    def __getattr__(self, item):
        default_value = self.CONFIGS.get(item, None)
        settings_value = getattr(settings, 'HOLONET_%s' % item, None)

        if settings_value is not None:
            return settings_value

        if default_value is not None:
            return default_value

        raise HolonetConfigrationError('Please set HOLONET_%s in your project settings.' % item)

holonet_settings = Settings()



# -*- coding: utf8 -*-

from django.test import TestCase

from holonet_django.validators import (ValidationError, is_prefix_unique, validate_email,
                                       validate_local_part, validate_prefix)


class ValidatorsTestCase(TestCase):

    def test_validate_email(self):
        self.assertTrue(validate_email('test@holonet.no'))

        self.assertFalse(validate_email('test@test@holonet.no'))

    def test_validate_local_part(self):
        self.assertTrue(validate_local_part('test'))

        self.assertFalse(validate_local_part('test@holonet.no'))

    def test_is_mapping_unique(self):
        self.assertTrue(is_prefix_unique('test', 'tests.Mapping1.1'))

    def test_validate_prefix(self):
        self.assertIsNone(validate_prefix('test_holonet'))
        self.assertRaises(ValidationError, validate_prefix, 'test@holonet.no')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
__version__ = '0.0.2'

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup

version = __version__

readme = open('README.rst').read()

setup(
    name='django-holonet',
    version=version,
    description="""This package is used to control holonet from a django project.""",
    long_description=readme,
    author='Abakus Webkom',
    author_email='webkom@abakus.no',
    url='https://github.com/webkom/django-holonet',
    packages=find_packages(exclude='tests'),
    include_package_data=True,
    install_requires=[
    ],
    license="MIT",
    zip_safe=False,
    keywords='django-holonet',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
    ],
)



# -*- coding: utf8 -*-
from django.db.models.fields.related import ReverseManyRelatedObjectsDescriptor
from django.db.models.loading import get_model
from django.db.models.signals import m2m_changed, post_delete, post_save, pre_save
from django.dispatch import receiver

from . import handlers, validators
from .exceptions import HolonetConfigrationError
from .settings import holonet_settings


@receiver(post_save, sender=get_model(*holonet_settings.RECIPIENT_MODEL.split('.')))
def recipient_change(instance, created, update_fields, *args, **kwargs):
    handlers.handle_recipient_change(instance, created, update_fields)


@receiver(post_delete, sender=get_model(*holonet_settings.RECIPIENT_MODEL.split('.')))
def recipient_delete(instance, *args, **kwargs):
    handlers.handle_recipient_delete(instance)


@receiver(pre_save, sender=get_model(*holonet_settings.RECIPIENT_MODEL.split('.')))
def pre_save_recipient_check(instance, *args, **kwargs):
    field = getattr(instance, holonet_settings.RECIPIENT_EMAIL_FIELD, None)
    validation = validators.validate_email(field)
    if not validation:
        raise ValueError('The email value is not valid.')


def mapping_change(instance, created, update_fields, *args, **kwargs):
    handlers.handle_mapping_change(instance, created, update_fields)


def mapping_delete(instance, *args, **kwargs):
    handlers.handle_mapping_delete(instance)


def mapping_recipient_list_change(instance, action, *args, **kwargs):
    if action.startswith('post'):
        handlers.handle_mapping_change(instance, created=False, updated_fields=None, force=True)


def pre_save_mapping_check(instance, *args, **kwargs):
    prefix = instance.mail_prefix
    validation = validators.validate_local_part(prefix)
    is_unique = validators.is_prefix_unique(prefix, instance.get_mapping_id)
    if not validation:
        raise ValueError('The mail prefix is not valid.')
    if not is_unique:
        raise ValueError('The selected mapping prefix is not unique.')


def add_signal_listeners():
    for table, properties in holonet_settings.MAPPING_MODELS.items():
        model = get_model(*table.split('.'))
        post_save.connect(mapping_change, model)
        post_delete.connect(mapping_delete, model)
        pre_save.connect(pre_save_mapping_check, model)

        # Add signal på many to many relations
        relation_list = properties.get('recipient_relations', [])
        if not isinstance(relation_list, list):
            raise HolonetConfigrationError('recipient_relations needs to be a list of strings.')

        for relation in relation_list:
            if not isinstance(relation, str):
                raise HolonetConfigrationError('Each item in recipient_relations need to be a '
                                               'string.')

            relation_field = getattr(model, relation, None)
            if relation_field is None:
                raise HolonetConfigrationError('Could not find the relation_field on the model.')

            if not isinstance(relation_field, ReverseManyRelatedObjectsDescriptor):
                raise HolonetConfigrationError('The relation field needs to be of type '
                                               'models.ManyToManyField')

            m2m_changed.connect(mapping_recipient_list_change, relation_field.through)

add_signal_listeners()



# -*- coding: utf8 -*-

from abc import abstractmethod

from django.contrib.contenttypes.models import ContentType
from django.db import models

from . import handlers
from .fields import MailPrefixField


class MailMapping(models.Model):

    mail_prefix = MailPrefixField()

    class Meta:
        abstract = True

    @abstractmethod
    def get_recipients(self):
        """
        Return all recipients of this mapping.
        Implement this function and return a list of recipients.
        :return: list of recipients. This needs to be of the same type as the RECIPIENT_MODEL class
        defined in settings.
        """
        raise NotImplementedError('Please impliment the get_recipients on the mapping model.')

    def force_mail_update(self):
        """
        This function forces update of the list. Use this if you don't store recipients in a
        many to many relation.
        :return: None
        """
        if self.pk:
            handlers.handle_mapping_change(self, created=False, updated_fields=None, force=True)

    def get_mapping_id(self):
        if not self.pk:
            raise ValueError('The model needs to have a pk before you can get the id.')
        return '%s.%s' % (ContentType.objects.get_for_model(self), self.pk)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django.contrib.contenttypes.models import ContentType
from django.test import TestCase
from mock import Mock

from holonet_django import handlers

from .models import Mapping1


class MappingModelTestCase(TestCase):

    def test_force_update(self):
        handlers.handle_mapping_change = Mock()

        mapping = Mapping1(mail_prefix='holonet_test')

        # Test no pk
        mapping.force_mail_update()
        self.assertEqual(handlers.handle_mapping_change.call_count, 0)

        # Test for saved element
        mapping.save()
        mapping.force_mail_update()
        # Use twise because
        handlers.handle_mapping_change.assert_called_twise_with(mapping, False, None, True)

    def test_mapping_id_no_pk(self):
        mapping = Mapping1(mail_prefix='holonet_test')
        self.assertRaises(ValueError, mapping.get_mapping_id)

    def test_mapping_id(self):
        mapping = Mapping1(mail_prefix='holonet_test')
        mapping.save()
        self.assertEquals(mapping.get_mapping_id(), '%s.%s' %
                          (ContentType.objects.get_for_model(mapping), mapping.pk))



# -*- coding: utf8 -*-

from django.db import models

from holonet_django.models import MailMapping


class TestRecipientModel(models.Model):

    username = models.CharField(max_length=100)
    email = models.EmailField()


class Mapping1(MailMapping):
    recipients = models.ManyToManyField(TestRecipientModel)

    invalid_test_field = 'Just a test value'

    def get_recipients(self):
        return self.recipients.all()


class Mapping2(MailMapping):

    def get_recipients(self):
        return None
